from django.contrib import admin

from models import YubicoKey


class YubicoKeyAdmin(admin.ModelAdmin):
    list_display = ('user', 'device_id', 'client_id', 'secret_key', 'enabled')
    search_fields = ['user', 'device_id', 'client_id']

admin.site.register(YubicoKey, YubicoKeyAdmin)



from django.db import models
from django.contrib.auth.models import User


class YubicoKey(models.Model):
    device_id = models.CharField(max_length=12)
    client_id = models.IntegerField()
    secret_key = models.CharField(max_length=80, blank=True)
    user = models.ForeignKey(User)
    enabled = models.BooleanField(default=True)

    class Meta:
        verbose_name = 'Yubico YubiKey'
        verbose_name_plural = 'Yubico YubiKeys'



from yubico.yubico import Yubico
from yubico.yubico_exceptions import YubicoError

from django.conf import settings
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User

from models import YubicoKey

# How much time can pass between the time when the first and last OTP is
# generated
YUBICO_MULTI_TIMEOUT = getattr(settings, 'YUBICO_MULTI_TIMEOUT', 10)


class YubicoBackend(object):
    """
    This backend requires a valid username and OTP from user to login.
    """
    def authenticate(self, username=None, otp=None):
        if not otp:
            return None

        count = len(otp)
        device_id = otp[0][:12]

        try:
            yubico = YubicoKey.objects.get(user__username=username,
                                           device_id=device_id)
        except YubicoKey.DoesNotExist:
            return None

        if not yubico.user.is_active or not yubico.enabled:
            return None

        secret_key = yubico.secret_key or None
        client = Yubico(yubico.client_id, secret_key)

        try:
            if count > 1:
                # More then 1 OTP provided, using multi mode
                status = client.verify_multi(otp_list=otp,
                                             max_time_window=
                                             YUBICO_MULTI_TIMEOUT)
            else:
                status = client.verify(otp[0])
        except YubicoError:
            return None

        if not status:
            return None

        return yubico.user

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None


class YubicoBackendStaff(ModelBackend):
    """
    If this backend is enabled, normal users can login with just a password
    but staff and super users are required to enter a valid OTP to log in.
    """
    def authenticate(self, *args, **kwargs):
        user = super(YubicoBackendStaff, self).authenticate(*args, **kwargs)

        if user and (user.is_staff or user.is_superuser):
            return None
        else:
            return user


class YubicoBackendRequireYubikey(ModelBackend):
    """
    If this backend is enabled, each user with at least one active YubiKey
    must enter a valid OTP to log in.
    """
    def authenticate(self, *args, **kwargs):
        user = super(YubicoBackendRequireYubikey, self).authenticate(*args,
                                                                     **kwargs)

        if user and user.yubicokey_set.filter(enabled=True).count():
            return None
        else:
            return user



from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.contrib.auth import login as auth_login
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from django.views.decorators.cache import never_cache

from forms import LoginForm, PasswordForm

# Ask for the user password after the token
YUBIKEY_USE_PASSWORD = getattr(settings, 'YUBICO_USE_PASSWORD', True)

# Name of the session key which stores user id
YUBIKEY_SESSION_USER_ID = getattr(settings, 'YUBICO_SESSION_USER_ID',
                                  'yubicodjango_user_id')

# Name of the session key which stores the name of the backend user used to log
# in.
YUBIKEY_SESSION_AUTH_BACKEND = getattr(settings, 'YUBICO_SESSION_AUTH_BACKEND',
                                       'yubicodjango_auth_backend')

# Name of the session key which stores attempt counter
YUBIKEY_SESSION_ATTEMPT_COUNTER = getattr(settings,
                                          'YUBIKEY_SESSION_ATTEMPT_COUNTER',
                                          'yubicodjango_counter')

# Name of the session key which stores number of password attemps
YUBIKEY_PASSWORD_ATTEMPTS = getattr(settings, 'YUBICO_PASSWORD_ATTEMPTS', 3)

# Django Yubico session keys
SESSION_KEYS = [YUBIKEY_SESSION_USER_ID, YUBIKEY_SESSION_AUTH_BACKEND,
                YUBIKEY_SESSION_ATTEMPT_COUNTER]


@never_cache
def login(request, template_name='django_yubico/login.html',
          redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Displays the login form and handles the login action.
    """
    redirect_to = request.REQUEST.get(redirect_field_name,
                                      settings.LOGIN_REDIRECT_URL)

    if request.method == 'POST':
        form = LoginForm(request.POST)

        if form.is_valid():
            user = form.user

            if YUBIKEY_USE_PASSWORD:
                # Dual factor authentication is enabled, user still needs to
                # enter his password
                request.session[YUBIKEY_SESSION_USER_ID] = user.pk
                request.session[YUBIKEY_SESSION_AUTH_BACKEND] = user.backend
                request.session[YUBIKEY_SESSION_ATTEMPT_COUNTER] = 1

                return HttpResponseRedirect(reverse('yubico_django_password'))
            else:
                auth_login(request=request, user=user)
                return HttpResponseRedirect(redirect_to)
    else:
        form = LoginForm()

    dictionary = {'form': form, redirect_field_name: redirect_to}
    return render_to_response(template_name, dictionary,
                              context_instance=RequestContext(request))


@never_cache
def password(request, template_name='django_yubico/password.html',
             redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Displays the password form and handles the login action.
    """

    redirect_to = request.REQUEST.get(redirect_field_name,
                                      settings.LOGIN_REDIRECT_URL)

    for key in SESSION_KEYS:
        # Make sure all the required session keys are present
        value = request.session.get(key, None)

        if value is None:
            return HttpResponseRedirect(reverse('yubico_django_login'))

    user_id = request.session[YUBIKEY_SESSION_USER_ID]
    auth_backend = request.session[YUBIKEY_SESSION_AUTH_BACKEND]

    user = User.objects.get(pk=user_id)
    user.backend = auth_backend

    if request.method == 'POST':
        form = PasswordForm(request.POST, user=user)

        if form.is_valid():
            auth_login(request=request, user=user)
            reset_user_session(session=request.session)
            return HttpResponseRedirect(redirect_to)
        else:
            # Limit the number of password attempts per token
            request.session[YUBIKEY_SESSION_ATTEMPT_COUNTER] += 1

            if request.session[YUBIKEY_SESSION_ATTEMPT_COUNTER] > \
               YUBIKEY_PASSWORD_ATTEMPTS:
                # Maximum number of attemps has been reached. Require user to
                # start from scratch.
                reset_user_session(session=request.session)
                return HttpResponseRedirect(reverse('yubico_django_login'))
    else:
        form = PasswordForm(user=user)

    dictionary = {'form': form, redirect_field_name: redirect_to}
    return render_to_response(template_name, dictionary,
                              context_instance=RequestContext(request))


def reset_user_session(session):
    """
    Remove all the Django Yubico related keys from the provided session.
    """
    for key in SESSION_KEYS:
        try:
            del session[key]
        except KeyError:
            pass



import re

from django import forms
from django.conf import settings
from django.contrib.auth import authenticate
from django.utils.translation import ugettext_lazy as _

RE_PUBLIC_ID = re.compile(r'^[cbdefghijklnrtuv]{12}$')
RE_TOKEN = re.compile(r'^[cbdefghijklnrtuv]{32,64}$')

# Multi mode (default = False)
YUBICO_MULTI_MODE = getattr(settings, 'YUBICO_MULTI_MODE', False)

# How many OTPs user needs to enter when multi mode is enabled
YUBICO_MULTI_NUMBER = getattr(settings, 'YUBICO_MULTI_NUMBER', 3)

STYLE = ''''background:url("/site_media/images/yubiright_16x16.gif") no-repeat
scroll 2px 2px white; padding-left:20px;'''
PASSWORD_INPUT_WIDGET_ATTRS = {'style': STYLE}


class LoginForm(forms.Form):
    username = forms.CharField(label=_('Username'))

    def __init__(self, *args, **kwargs):
        super(LoginForm, self).__init__(*args, **kwargs)
        self.user = None

        min_length = 34
        max_length = 64

        if not YUBICO_MULTI_MODE:
            widget = forms.PasswordInput(attrs=PASSWORD_INPUT_WIDGET_ATTRS)
            self.fields['otp'] = forms.RegexField(label=_('OTP'),
                                                  widget=widget,
                                                  regex=RE_TOKEN,
                                                  min_length=min_length,
                                                  max_length=max_length)
        else:
            for index in range(0, YUBICO_MULTI_NUMBER):
                label = _('OTP %d' % (index + 1))
                widget = forms.PasswordInput(attrs=PASSWORD_INPUT_WIDGET_ATTRS)

                field = forms.RegexField(label=label, widget=widget,
                                         regex=RE_TOKEN,
                                         min_length=min_length,
                                         max_length=max_length)
                self.fields['otp_%d' % index] = field

    def clean(self):
        username = self.cleaned_data.get('username')

        otp_list = []
        if not YUBICO_MULTI_MODE:
            otp = self.cleaned_data.get('otp')
            otp_list.append(otp)
        else:
            for index in range(0, YUBICO_MULTI_NUMBER):
                otp = self.cleaned_data.get('otp_%d' % index)
                otp_list.append(otp)

        if username and otp_list:
            self.user = authenticate(username=username, otp=otp_list)

        if self.user is None:
            raise forms.ValidationError(_('The provided OTP is invalid.'))

        return self.cleaned_data


class PasswordForm(forms.Form):
    password = forms.CharField(label=_('Password'),
                               widget=forms.PasswordInput(),
                               required=True)

    def __init__(self, *args, **kwargs):
        self.user = kwargs['user']
        del(kwargs['user'])
        super(PasswordForm, self).__init__(*args, **kwargs)

    def clean_password(self):
        password = self.cleaned_data['password']

        if not self.user.check_password(password):
            msg = _('The provided password is incorrect')
            raise forms.ValidationError(msg)



try:
    from django.conf.urls import patterns, url
except ImportError:
    # Django < 1.4
    from django.conf.urls.defaults import patterns, url

from views import login, password

urlpatterns = patterns(
    '',
    url(r'^login', login, name='yubico_django_login'),
    url(r'^password', password, name='yubico_django_password'),
)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
import os
import re
from setuptools import setup

version_re = re.compile(
    r'__version__ = (\(.*?\))')

cwd = os.path.dirname(os.path.abspath(__file__))
fp = open(os.path.join(cwd, 'django_yubico', '__init__.py'))

version = None
for line in fp:
    match = version_re.search(line)
    if match:
        version = eval(match.group(1))
        break
else:
    raise Exception('Cannot find version in __init__.py')
fp.close()

setup(
    name='django_yubico',
    version='.' . join(map(str, version)),
    description='Yubico Yubikey authentication backend for Django',
    author='Tomaz Muraus',
    author_email='tomaz+pypi@tomaz.me',
    license='BSD',
    url='http://github.com/Kami/django-yubico-authentication-backend/',
    packages=['django_yubico'],
    install_requires=[
        'yubico-client>=1.7,<1.8'
    ],
    provides=['django_yubico'],
    package_data={
        'django_yubico': [
            'site_media/images/*',
            'templates/django_yubico/*.html'
        ]
    },

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries',
    ],
)
